/**
 * @file        ADSO-9566
 * @description Create finite capacity stocking point
 * @author      Chan Clarence (ccn7@3ds.com)
 * @copyright   Dassault Systemes
 */
import { qCustomMatcher } from '../../e2elib/lib/src/main/qmatchers.const';
import { AppMP, Demo, Scenario } from '../../libmp/appmp';
import { DataFoodBaseEntityName } from '../../libmp/data/data.navigation';
import { DialogStockingPoint, DialogStockingPointFields } from '../../libmp/dialogs/dialog.stockingpoint';
import { StepList } from '../../libappsop/listsop';
import { StepDialog } from '../../libappsop/dialogsop';
import { ListEntityColumn, ListEntity, ListEntityContextMenuItem } from '../../libmp/forms/form.unit';
import { DialogUnit } from '../../libmp/dialogs/dialog.unit';
import { DataStockingPointCapacityType } from '../../libmp/data/data.stockingpoint';
import { StepMatrix } from '../../libappsop/matrixeditorsop';
import { MatrixEditorStockingPointCapacities } from '../../libmp/forms/form.stockingpointcapacities';

describe('ADSO-9566 - Create finite capacity stocking point', () => {
  const appMP = AppMP.getInstance();
  let dlgStockingPoint: DialogStockingPoint;
  const listEntity = appMP.viewEntity.formEntity.listEntity;
  const newSPName = 'New finite stocking point';
  const newSPNameWithUoM = `${newSPName} (MT)`;
  const selectRowValues: ListEntityColumn = { Name: DataFoodBaseEntityName.Suppliers };
  const verifyDlgValues: DialogStockingPointFields = { Parent: DataFoodBaseEntityName.Suppliers, Name: '', ID: '', IsInfiniteCapacity: 'true' }; // Default checked infinite capacity
  const verifyDlgValues2: DialogStockingPointFields = { ID: newSPName };
  const updateDlgValues: DialogStockingPointFields = { Name: newSPName, IsInfiniteCapacity: 'false' };
  const verifySPCreatedInList: ListEntityColumn = { Name: newSPName, 'Capacity type': DataStockingPointCapacityType.Limited };
  const verifySPParentInList: ListEntityColumn[] = [{ Name: DataFoodBaseEntityName.Europe }, { Name: DataFoodBaseEntityName.Suppliers }];

  beforeAll(async () => {
    jasmine.addMatchers(qCustomMatcher);
    await appMP.login();
  });

  afterAll(async () => {
    await appMP.cleanupAndLogout();
  });

  afterEach(async () => {
    await appMP.checkToastMessage();
  });

  it(`Setup - ${AppMP.getDemoDataPath(Demo.Food, Scenario.Base)}`, async () => {
    await appMP.createDemoDataset(Demo.Food, Scenario.Base, false);
  });

  it(`Setup - Open view ${appMP.viewEntity.viewPath}.`, async () => {
    await appMP.viewEntity.switchTo();
  });

  it(`Step 1 - ${StepList.selectRowAndClickMenu(ListEntity.title, selectRowValues, ListEntityContextMenuItem.CreateStockingPoint.Label)}`, async () => {
    const supplierRow = await listEntity.getRow(selectRowValues, [{Name: DataFoodBaseEntityName.Europe}]);
    [, dlgStockingPoint] = await listEntity.selectContextMenu(ListEntityContextMenuItem.CreateStockingPoint, supplierRow, new DialogStockingPoint()) as [DialogUnit, DialogStockingPoint];
  });

  it(`Step 2 - ${StepDialog.verifyDialogValues(DialogStockingPoint.title, verifyDlgValues)}`, async () => {
    await dlgStockingPoint.verifyDialogValues(verifyDlgValues);
  });

  it(`Step 3 - ${StepDialog.updateDialogValues(DialogStockingPoint.title, updateDlgValues)} ${StepDialog.verifyDialogValues(DialogStockingPoint.title, verifyDlgValues2)} ${StepDialog.clickOK()}`, async () => {
    await dlgStockingPoint.updateDialogValues(updateDlgValues);
    await dlgStockingPoint.verifyDialogValues(verifyDlgValues2); // ID refreshed = user entered Name
    await dlgStockingPoint.clickOK();
  });

  it(`Step 4 - ${StepList.verifyRowExists(ListEntity.title, verifySPCreatedInList, verifySPParentInList)}`, async () => {
    await listEntity.verifyRowExists(verifySPCreatedInList, verifySPParentInList);

    // Reset current view before opening next view
    await appMP.resetActiveView(appMP.viewEntity);
  });

  it(`Step 5 - Open view ${appMP.viewCapacitiesStockingPoints.viewPath}.`, async () => {
    await appMP.viewCapacitiesStockingPoints.switchTo();
  });

  it(`Step 6 - ${StepMatrix.verifyRowExists(MatrixEditorStockingPointCapacities.title, newSPNameWithUoM)}`, async () => {
    await appMP.viewCapacitiesStockingPoints.formStockingPointCapacities.meStockingPointCapacities.verifyRowExist(newSPNameWithUoM);
    await appMP.resetActiveView(appMP.viewCapacitiesStockingPoints);
  });
});
